@@ -1357,8 +1357,7 @@ bool pkgProblemResolver::ResolveByKeepInternal()
 	       pkgCache::PkgIterator Pkg = Ver.ParentPkg();
 	       
 	       // It is not keepable
-	       if (Cache[Pkg].InstallVer == 0 ||
-		   Pkg->CurrentVer == 0)
+	       if (Pkg->CurrentVer == 0)
 		  continue;
 
 	       if (not Cache[Pkg].Protect())
@@ -1401,6 +1400,9 @@ bool pkgProblemResolver::ResolveByKeepInternal()
 
    delete[] PList;
 
+   if (Cache.BrokenCount() != 0)
+      return _error->Error(_("Unable to correct problems, you have held broken packages."));
+
    if (Debug)
       Cache.CheckConsistency("keep done");
 
