@@ -286,6 +286,13 @@ void pkgPolicy::CreatePin(pkgVersionMatch::MatchType Type,string Name,
 // Returns true if this update is excluded by phasing.
 static inline bool ExcludePhased(std::string machineID, pkgCache::VerIterator const &Ver)
 {
+   if (Ver.PhasedUpdatePercentage() == 100)
+      return false;
+
+   // FIXME: We have migrated to a legacy implementation until LP: #1929082 is fixed
+   if (not _config->FindB("APT::Get::Phase-Policy", false))
+      return false;
+
    // The order and fallbacks for the always/never checks come from update-manager and exist
    // to preserve compatibility.
    if (_config->FindB("APT::Get::Always-Include-Phased-Updates",
@@ -310,11 +317,9 @@ static inline bool ExcludePhased(std::string machineID, pkgCache::VerIterator co
 }
 APT_PURE signed short pkgPolicy::GetPriority(pkgCache::VerIterator const &Ver, bool ConsiderFiles)
 {
-   if (Ver.PhasedUpdatePercentage() != 100)
-   {
-      if (ExcludePhased(d->machineID, Ver))
-	 return 1;
-   }
+   auto ceiling = std::numeric_limits<signed int>::max();
+   if (ExcludePhased(d->machineID, Ver))
+      ceiling = 1;
    if (VerPins[Ver->ID].Type != pkgVersionMatch::None)
    {
       // If all sources are never pins, the never pin wins.
@@ -322,10 +327,10 @@ APT_PURE signed short pkgPolicy::GetPriority(pkgCache::VerIterator const &Ver, b
 	 return NEVER_PIN;
       for (pkgCache::VerFileIterator file = Ver.FileList(); file.end() == false; file++)
 	 if (GetPriority(file.File()) != NEVER_PIN)
-	    return VerPins[Ver->ID].Priority;
+	    return std::min((int)VerPins[Ver->ID].Priority, ceiling);
    }
    if (!ConsiderFiles)
-      return 0;
+      return std::min(0, ceiling);
 
    // priorities are short ints, but we want to pick a value outside the valid range here
    auto priority = std::numeric_limits<signed int>::min();
@@ -342,7 +347,7 @@ APT_PURE signed short pkgPolicy::GetPriority(pkgCache::VerIterator const &Ver, b
 	 priority = std::max<decltype(priority)>(priority, GetPriority(file.File()));
    }
 
-   return priority == std::numeric_limits<decltype(priority)>::min() ? 0 : priority;
+   return std::min(priority == std::numeric_limits<decltype(priority)>::min() ? 0 : priority, ceiling);
 }
 APT_PURE signed short pkgPolicy::GetPriority(pkgCache::PkgFileIterator const &File)
 {
